﻿
using System.Data;
using System.IO.Compression;
using Microsoft.AspNetCore.Hosting;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using SqlSugar;
using Tiobon.Core.Common;
using Tiobon.Core.Common.Caches;
using Tiobon.Core.IRepository.Base;
using Tiobon.Core.IServices;
using Tiobon.Core.Model;
using Tiobon.Core.Model.Models;
using Tiobon.Core.Services.BASE;

namespace Tiobon.Core.Services;

/// <summary>
/// 课件 (服务)
/// </summary>
public class Ghre_CourseWareServices : BaseServices<Ghre_CourseWare, Ghre_CourseWareDto, InsertGhre_CourseWareInput, EditGhre_CourseWareInput>, IGhre_CourseWareServices
{
    private readonly IBaseRepository<Ghre_CourseWare> _dal;
    private IGhre_CourseServices _ghre_CourseServices;
    private readonly IWebHostEnvironment _hostingEnvironment;
    private IGhre_CourseWareAttachmentServices _ghre_CourseWareAttachmentServices;

    public Ghre_CourseWareServices(ICaching caching,
        IGhre_CourseServices ghre_CourseServices,
        IWebHostEnvironment hostingEnvironment,
        IGhre_CourseWareAttachmentServices ghre_CourseWareAttachmentServices,
        IBaseRepository<Ghre_CourseWare> dal)
    {
        this._dal = dal;
        base.BaseDal = dal;
        base._caching = caching;
        _ghre_CourseServices = ghre_CourseServices;
        _ghre_CourseWareAttachmentServices = ghre_CourseWareAttachmentServices;
        _hostingEnvironment = hostingEnvironment;
    }

    public override async Task<long> Add(InsertGhre_CourseWareInput entity)
    {
        if (entity.CourseIds2.Any())
        {
            entity.CourseIds = JsonConvert.SerializeObject(entity.CourseIds2);
            var courses = await _ghre_CourseServices.Query(x => entity.CourseIds2.Contains(x.Id));
            entity.CourseNames = string.Join(",", courses.Select(x => x.CourseName));
        }
        var result = await base.Add(entity);
        entity.Attachments.ForEach(x => x.CourseWareId = result);
        await _ghre_CourseWareAttachmentServices.Add(entity.Attachments);
        return result;
    }

    public override async Task<bool> Update(long Id, EditGhre_CourseWareInput editModel)
    {
        if (editModel.CourseIds2.Any())
        {
            editModel.CourseIds = JsonConvert.SerializeObject(editModel.CourseIds2);
            var courses = await _ghre_CourseServices.Query(x => editModel.CourseIds2.Contains(x.Id));
            editModel.CourseNames = string.Join(",", courses.Select(x => x.CourseName));
        }

        await _ghre_CourseWareAttachmentServices.Delete(x => x.CourseWareId == Id);
        editModel.Attachments.ForEach(x => x.CourseWareId = Id);
        await _ghre_CourseWareAttachmentServices.Add(editModel.Attachments);


        return await base.Update(Id, editModel);
    }

    public override async Task<ServiceFormResult<Ghre_CourseWareDto>> QueryForm(QueryForm body)
    {
        var result = await base.QueryForm(body);
        string courseIds = result.result.DT_TableDataT1[0].CourseIds;
        if (!string.IsNullOrWhiteSpace(courseIds))
        {
            result.result.DT_TableDataT1[0].CourseIds2 = JsonConvert.DeserializeObject<List<long>>(courseIds);
            if (result.result.DT_TableDataT1[0].CourseIds2.Any())
            {

            }
        }

        else result.result.DT_TableDataT1[0].CourseIds2 = new List<long>();

        result.result.DT_TableDataT1[0].Attachments = await _ghre_CourseWareAttachmentServices.Query(x => x.CourseWareId == body.id);
        var IDS = result.result.DT_TableDataT1[0].CourseIds2;
        result.result.DT_TableDataT1[0].Courses = await _ghre_CourseServices.Query(x => IDS.Contains(x.Id));

        if (body.doType == "Copy")
        {
            result.result.DT_TableDataT1[0].CourseWareNo = null;
            result.result.DT_TableDataT1[0].CourseWareName = null;
        }



        //        string sql = "SELECT Id,QuestionId from Ghre_QuestionAnswer where AnswerContent='单选题'";
        //        var questionAnswers = await Db.Ado.SqlQueryAsync<Ghre_QuestionAnswer>(sql);
        //        for (int i = 0; i < questionAnswers.Count; i++)
        //        {
        //            await Db.Deleteable(new Ghre_QuestionAnswer() { Id = questionAnswers[i].Id }).ExecuteCommandAsync();
        //            sql = $@"UPDATE A
        //SET A.TaxisNo = B.NUM
        //FROM Ghre_QuestionAnswer A
        //     JOIN
        //     (SELECT ID, NUM * 100 NUM
        //      FROM (SELECT *, ROW_NUMBER () OVER (ORDER BY TaxisNo) NUM
        //            FROM (SELECT *
        //                  FROM (SELECT *
        //                        FROM Ghre_QuestionAnswer
        //                        WHERE QuestionId = '{questionAnswers[i].QuestionId}') A) B) C) B
        //        ON A.ID = b.ID;



        //UPDATE Ghre_QuestionAnswer
        //SET QuestionNo =
        //       CASE TaxisNo
        //          WHEN 100 THEN 'A'
        //          WHEN 200 THEN 'B'
        //          WHEN 300 THEN 'C'
        //          WHEN 400 THEN 'D'
        //          WHEN 500 THEN 'E'
        //          WHEN 600 THEN 'F'
        //          WHEN 700 THEN 'G'
        //          WHEN 800 THEN 'H'
        //          WHEN 900 THEN 'I'
        //          WHEN 1000 THEN 'J'
        //          WHEN 1100 THEN 'K'
        //          WHEN 1200 THEN 'L'
        //       END
        //WHERE QuestionId = '{questionAnswers[i].QuestionId}'";
        //            await Db.Ado.SqlQueryAsync<Ghre_QuestionAnswer>(sql);
        //        }

        return result;

    }

    public override async Task<ServicePageResult<Ghre_CourseWareDto>> QueryFilterPage(QueryBody filter)
    {
        bool? IsAllowDownload = null;
        if (filter.jsonParam != null)
            foreach (JProperty jProperty in filter.jsonParam.Properties())
            {
                var name = jProperty.Name;
                var value = jProperty.Value.ToString();
                if (name != "IsAllowDownload")
                    continue;
                if (!string.IsNullOrWhiteSpace(value))
                {
                    var jsonParam = JsonConvert.DeserializeObject<JsonParam>(value);

                    switch (name)
                    {
                        case "IsAllowDownload":
                            IsAllowDownload = Convert.ToBoolean(jsonParam.columnValue);
                            break;
                        default:
                            break;
                    }
                }
            }
        string condition = string.Empty;
        if (IsAllowDownload != null)
        {
            string sql = @$"SELECT DISTINCT B.Id
                                FROM Ghre_CourseWareAttachment A
                                     JOIN Ghre_CourseWare B ON A.CourseWareId = B.Id AND B.IsEnable = 1
                                WHERE A.IsEnable = 1 AND A.IsAllowDownload = '{IsAllowDownload}'";
            var entitys = await Db.Ado.SqlQueryAsync<long>(sql);
            if (entitys.Any())
                condition = "Id IN (" + string.Join(",", entitys.Select(x => x)) + ")";
        }
        var result = await base.QueryFilterPage(filter, condition);
        var data = result.result.DT_TableDataT1;
        var attachmentIds = data.Select(x => x.Id).ToList();
        var attachments = await _ghre_CourseWareAttachmentServices.Query(x => x.CourseWareId != null && attachmentIds.Contains(x.CourseWareId.Value));
        data.ForEach(async x =>
        {
            if (!string.IsNullOrWhiteSpace(x.CourseIds))
                x.CourseIds2 = JsonConvert.DeserializeObject<List<long>>(x.CourseIds);

            x.StudyDuration = $"{x.Hours}小时{x.Minutes}分钟";
            x.SourceLabel = await GetParaLabel("CourseWareSource", x.Source);
            x.Attachments = attachments.Where(a => a.CourseWareId == x.Id).ToList();
        });
        result.result.DT_TableDataT1 = data;
        return result;

    }


    public async Task<ServiceResult<string>> DownZip(long id)
    {
        var result = await base.QueryById(id);
        if (result is null)
            return ServiceResult<string>.OprateFailed("无效的课件ID！");
        var attachments = await _ghre_CourseWareAttachmentServices.Query(x => x.CourseWareId == id);
        var webRootPath = _hostingEnvironment.WebRootPath;
        var outPath = $"/files/upload/{result.CourseWareName}_{DateTime.Now.ToString("yyyyMMddHHmmss")}.zip";
        var files = attachments.Select(x => $"{webRootPath}{"\\" + x.RelativePath}").ToArray();
        if (files.Length > 0)
        {
            if (files.Length > 1)
                CreateZip($"{webRootPath}{outPath}", files);
            else
                outPath = attachments[0].RelativePath;
            return ServiceResult<string>.OprateSuccess(null, outPath);
        }
        else
            return ServiceResult<string>.OprateFailed("该课件无附件！");

    }

    public static void CreateZip(string zipPath, string[] filesToZip)
    {
        if (File.Exists(zipPath))
        {
            throw new IOException("File already exists.");
        }

        using (FileStream zipFile = new FileStream(zipPath, FileMode.Create))
        {
            using (ZipArchive zipArchive = new ZipArchive(zipFile, ZipArchiveMode.Create))
            {
                foreach (string file in filesToZip)
                {
                    if (File.Exists(file))
                    {
                        ZipArchiveEntry entry = zipArchive.CreateEntryFromFile(file, Path.GetFileName(file));
                    }
                }
            }
        }
    }

    #region 删除
    // <summary>
    /// 删除指定ID的数据
    /// </summary>
    /// <param name="id">主键ID</param>
    /// <returns></returns>
    public override async Task<bool> DeleteById1(object id)
    {

        if (await Db.Queryable<Ghre_Course>().AnyAsync(x => x.CourseWareId == (long)id))
            throw new Exception($"该课件已与课程关联,暂不可删除！");

        return await base.DeleteById1(id);
    }

    /// <summary>
    /// 删除指定ID集合的数据(批量删除)
    /// </summary>
    /// <param name="ids">主键ID集合</param>
    /// <returns></returns>
    public override async Task<bool> DeleteByIds1(long[] ids)
    {

        foreach (var id in ids)
        {
            if (await Db.Queryable<Ghre_Course>().AnyAsync(x => x.CourseWareId == (long)id))
                throw new Exception($"课件已与课程关联,暂不可删除！");
        }

        return await base.DeleteByIds1(ids);
    }
    #endregion
}